@@ -21,8 +21,8 @@
  *
  * CVS Information:
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/globus_gram_job_manager.h,v $
- * $Date: 2010/06/10 12:49:14 $
- * $Revision: 1.47.2.9 $
+ * $Date: 2011/01/26 19:54:57 $
+ * $Revision: 1.47.2.12 $
  * $Author: bester $
  */
 #ifndef GLOBUS_GRAM_JOB_MANAGER_INCLUDE
@@ -476,6 +476,11 @@ typedef struct globus_gram_job_manager_s
     globus_callback_handle_t            pending_restart_handle;
     /** Usage stats tracking data */
     globus_i_gram_usage_tracker_t *     usagetracker;
+    /**
+     * Error message extension for early misconfiguration-type errors that
+     * occur before a request is read.
+     */
+    char *                              gt3_failure_message;
 }
 globus_gram_job_manager_t;
 
@@ -1311,7 +1316,7 @@ globus_gram_job_manager_init_seg(
 
 globus_result_t
 globus_gram_job_manager_shutdown_seg(
-    const char *                        seg_module);
+    globus_gram_job_manager_t *         manager);
 
 void
 globus_gram_job_manager_seg_handle_event(
@@ -1508,12 +1513,16 @@ globus_i_gram_usage_stats_destroy(
     globus_gram_job_manager_t *         manager);
 
 /* startup_socket.c */
+int
+globus_gram_job_manager_startup_lock(
+    globus_gram_job_manager_t *         manager,
+    int *                               lock_fd);
+
 int
 globus_gram_job_manager_startup_socket_init(
     globus_gram_job_manager_t *         manager,
     globus_xio_handle_t *               handle,
-    int *                               socket_fd,
-    int *                               lock_fd);
+    int *                               socket_fd);
 
 int
 globus_gram_job_manager_starter_send(
