@@ -21,8 +21,8 @@
  * CVS Information:
  * 
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/globus_gram_job_manager_config.c,v $
- * $Date: 2010/06/10 12:49:14 $
- * $Revision: 1.5.2.4 $
+ * $Date: 2011/01/26 01:35:37 $
+ * $Revision: 1.5.2.6 $
  * $Author: bester $
  */
 
@@ -295,8 +295,11 @@ globus_gram_job_manager_config_init(
         {
             char *                  log_level_string = strdup(argv[++i]);
             char *                  level_string = NULL;
+            char *                  last_string = NULL;
 
-            while ((level_string = strsep(&log_level_string, "|")) != NULL)
+            for (level_string = strtok_r(log_level_string, "|", &last_string);
+                 level_string != NULL;
+                 level_string = strtok_r(NULL, "|", &last_string))
             {
                 if (strcmp(level_string, "FATAL") == 0)
                 {
@@ -323,6 +326,7 @@ globus_gram_job_manager_config_init(
                     config->log_levels |= GLOBUS_GRAM_JOB_MANAGER_LOG_TRACE;
                 }
             }
+            free(log_level_string);
         }
         else if (strcmp(argv[i], "-disable-usagestats") == 0)
         {
@@ -440,7 +444,6 @@ globus_gram_job_manager_config_init(
             goto out;
         }
     }
-
     /* Now initialize values from our environment */
     config->home = strdup(getenv("HOME"));
     if (config->home == NULL)
@@ -479,6 +482,18 @@ globus_gram_job_manager_config_init(
             goto out;
         }
     }
+    if (config->job_state_file_dir == NULL)
+    {
+        config->job_state_file_dir = globus_common_create_string(
+                "%s/tmp/gram_job_state/",
+                config->globus_location);
+        if (config->job_state_file_dir == NULL)
+        {
+            rc = GLOBUS_GRAM_PROTOCOL_ERROR_MALLOC_FAILED;
+
+            goto out;
+        }
+    }
     if (config->scratch_dir_base == NULL)
     {
         config->scratch_dir_base = strdup(
